@@ -1,6 +1,5 @@
 /* BFD back-end for i386 a.out binaries.
-   Copyright 1990, 1991, 1992, 1994, 1996, 1997, 2001, 2002, 2003, 2005,
-   2007 Free Software Foundation, Inc.
+   Copyright (C) 1990-2024 Free Software Foundation, Inc.
 
    This file is part of BFD, the Binary File Descriptor library.
 
@@ -27,19 +26,18 @@
    system, and I'll stick it in for the next release.  */
 
 #define N_HEADER_IN_TEXT(x)	0
-#define N_TXTOFF(x)   		0x20
-#define N_TXTADDR(x) 		(N_MAGIC (x) == ZMAGIC ? 0x1020 : 0)
-#define N_TXTSIZE(x) 		((x).a_text)
-#define TARGET_PAGE_SIZE 	4096
-#define SEGMENT_SIZE 		0x400000
-#define DEFAULT_ARCH 		bfd_arch_i386
+#define N_TXTOFF(x)		0x20
+#define N_TXTADDR(x)		(N_MAGIC (x) == ZMAGIC ? 0x1020 : 0)
+#define N_TXTSIZE(x)		((x)->a_text)
+#define TARGET_PAGE_SIZE	4096
+#define SEGMENT_SIZE		0x400000
+#define DEFAULT_ARCH		bfd_arch_i386
 
 /* Do not "beautify" the CONCAT* macro args.  Traditional C will not
    remove whitespace added here, and thus will fail to concatenate
    the tokens.  */
-#define MY(OP) CONCAT2 (i386aout_,OP)
+#define MY(OP) CONCAT2 (i386_aout_,OP)
 #define TARGETNAME "a.out-i386"
-#define NO_WRITE_HEADER_KLUDGE 1
 
 #include "sysdep.h"
 #include "bfd.h"
@@ -49,23 +47,23 @@
 
 /* Set the machine type correctly.  */
 
-static bfd_boolean
+static bool
 i386aout_write_object_contents (bfd *abfd)
 {
   struct external_exec exec_bytes;
   struct internal_exec *execp = exec_hdr (abfd);
 
-  N_SET_MACHTYPE (*execp, M_386);
+  N_SET_MACHTYPE (execp, M_386);
 
   obj_reloc_entry_size (abfd) = RELOC_STD_SIZE;
 
   WRITE_HEADERS (abfd, execp);
 
-  return TRUE;
+  return true;
 }
 
 #define MY_write_object_contents  i386aout_write_object_contents
-#define MY_backend_data           & MY (backend_data)
+#define MY_backend_data		  & MY (backend_data)
 
 static const struct aout_backend_data MY (backend_data);
 
