@@ -1,4 +1,4 @@
-/* Copyright (C) 2021 Free Software Foundation, Inc.
+/* Copyright (C) 2021-2024 Free Software Foundation, Inc.
    Contributed by Oracle.
 
    This file is part of GNU Binutils.
@@ -112,15 +112,14 @@ extern "C"
     int timecvt;        /* multiplier to convert metric to time, 0 if N/A */
     ABST_type memop;    /* type of backtracking allowed */
     char *short_desc;   /* optional one-liner description, or NULL */
-    int type;           /* Type of perf_event_attr */
-    long long config;   /* perf_event_type -specific configuration */
     /* the fields above this line are expected, in order, by the tables in hwctable.c */
     /* ================================================== */
     /* the fields below this line are more flexible */
+    unsigned int use_perf_event_type : 16; /* Set 1 to use two fields below */
+    unsigned int type : 16; /* Type of perf_event_attr */
+    long long config;   /* perf_event_type -specific configuration */
+    long long config1;  /* perf_event_type -specific configuration */
     int sort_order;     /* "tag" to associate experiment record with HWC def */
-    regno_t *reg_list;  /* if not NULL, legal values for <reg_num> field above */
-    /* Note: reg_list will be terminated by REGNO_ANY */
-    /* Max size of array is MAX_PICS */
     hrtime_t min_time;  /* target minimum time between overflow events.  0 is off.  See HWCTIME_* macros */
     hrtime_t min_time_default; /* if min_time==HWCTIME_AUTO, use this value instead.  0 is off. */
     int ref_val;    /* if min_time==HWCTIME_AUTO, use this time.  0 is off. */
