@@ -1,5 +1,5 @@
 #!/usr/bin/env bash
-#   Copyright (C) 1990-2018 Free Software Foundation
+#   Copyright (C) 1990-2024 Free Software Foundation
 #
 # This file is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -26,10 +26,12 @@ BZIPPROG=bzip2
 GZIPPROG=gzip
 LZIPPROG=lzip
 XZPROG=xz
-MD5PROG=md5sum
+ZSTDPROG=zstd
+SHA256PROG=sha256sum
 MAKE=make
 CC=gcc
 CXX=g++
+release_date=
 
 # Default to avoid splitting info files by setting the threshold high.
 MAKEINFOFLAGS=--split-size=5000000
@@ -44,12 +46,15 @@ DEVO_SUPPORT="ar-lib ChangeLog compile config config-ml.in config.guess \
 	ltmain.sh ltoptions.m4 ltsugar.m4 ltversion.m4 lt~obsolete.m4 \
 	MAINTAINERS Makefile.def Makefile.in Makefile.tpl missing mkdep \
 	mkinstalldirs move-if-change README README-maintainer-mode \
-	src-release.sh symlink-tree test-driver ylwrap"
+	SECURITY.txt src-release.sh symlink-tree test-driver ylwrap \
+        multilib.am ChangeLog.git"
 
 # Files in devo/etc used in any net release.
-ETC_SUPPORT="Makefile.in configure configure.in standards.texi \
-	make-stds.texi standards.info* configure.texi configure.info* \
-	ChangeLog configbuild.* configdev.* fdl.texi texi2pod.pl gnu-oids.texi"
+ETC_SUPPORT="ChangeLog Makefile.am Makefile.in aclocal.m4 add-log.el \
+	add-log.vi configbuild.* configdev.* configure configure.ac \
+	configure.in configure.info* configure.texi fdl.texi gnu-oids.texi \
+	make-stds.texi standards.info* standards.texi texi2pod.pl \
+	update-copyright.py"
 
 # Get the version number of a given tool
 getver()
@@ -61,17 +66,63 @@ getver()
 	$tool/common/create-version.sh $tool 'dummy-host' 'dummy-target' VER.tmp
 	cat VER.tmp | grep 'version\[\]' | sed 's/.*"\([^"]*\)".*/\1/' | sed 's/-git$//'
         rm -f VER.tmp
-    elif test -f $tool/gdbsupport/create-version.sh; then
-	$tool/gdbsupport/create-version.sh $tool 'dummy-host' 'dummy-target' VER.tmp
+    elif test $tool = "gdb"; then
+	./gdbsupport/create-version.sh $tool 'dummy-host' 'dummy-target' VER.tmp
 	cat VER.tmp | grep 'version\[\]' | sed 's/.*"\([^"]*\)".*/\1/' | sed 's/-git$//'
         rm -f VER.tmp
     elif test -f $tool/version.in; then
-	head -1 $tool/version.in
+	head -n 1 $tool/version.in
     else
 	echo VERSION
     fi
 }
 
+clean_sources()
+{
+    # Check that neither staged nor unstaged changes of any tracked file remains.
+    if [ -n "$(git status --porcelain -uno)" ]; then
+        echo "There are uncommitted changes. Please commit or stash them."
+        exit 1
+    fi
+
+    echo "==> Cleaning sources."
+
+    # Remove all untracked files.
+    git clean -fdx
+
+    # Umask for any new file created by this script.
+    umask 0022
+
+    # Fix permissions of all tracked files and directories according to the previously set umask.
+    echo "==> Fixing permissions."
+    permreg=$(printf "%o" $((0666 & ~$(umask))))
+    permexe=$(printf "%o" $((0777 & ~$(umask))))
+    git ls-tree -rt --format "objectmode=%(objectmode) path=%(path)" HEAD | while read -r objectprop; do
+        eval $objectprop
+        case $objectmode in
+            100644)
+                # regular file
+                chmod $permreg $path
+                ;;
+            100755|040000)
+                # executable file or directory
+                chmod $permexe $path
+                ;;
+            120000)
+                # symlink, do nothing, always lrwxrwxrwx.
+                ;;
+            160000)
+                # submodule, currently do nothing
+                ;;
+            *)
+                # unlikely, might be a future version of Git
+                echo unsupported object at $path
+                exit 1
+                ;;
+        esac
+    done
+}
+
 # Setup build directory for building release tarball
 do_proto_toplev()
 {
@@ -80,8 +131,7 @@ do_proto_toplev()
     tool=$3
     support_files=$4
 
-    echo "==> Cleaning sources."
-    find \( -name "*.orig" -o  -name "*.rej" -o -name "*~" -o -name ".#*" -o -name "*~$bkpat" \) -exec rm {} \;
+    clean_sources
     
     echo "==> Making $package-$ver/"
     # Take out texinfo from a few places.
@@ -93,7 +143,7 @@ do_proto_toplev()
     # built in the gold dir.  The disables speed the build a little.
     enables=
     disables=
-    for dir in binutils gas gdb gold gprof ld libctf libdecnumber readline sim; do
+    for dir in binutils gas gdb gold gprof gprofng libsframe ld libctf libdecnumber readline sim; do
 	case " $tool $support_files " in
 	    *" $dir "*) enables="$enables --enable-$dir" ;;
 	    *) disables="$disables --disable-$dir" ;;
@@ -126,57 +176,54 @@ do_proto_toplev()
 	    fi
 	else
 	    if (echo x$d | grep / >/dev/null); then
-	      mkdir -p proto-toplev/`dirname $d`
-	      x=`dirname $d`
-	      ln -s ../`echo $x/ | sed -e 's,[^/]*/,../,g'`$d proto-toplev/$d
+		mkdir -p proto-toplev/`dirname $d`
+		x=`dirname $d`
+		ln -s ../`echo $x/ | sed -e 's,[^/]*/,../,g'`$d proto-toplev/$d
 	    else
-	      ln -s ../$d proto-toplev/$d
+		ln -s ../$d proto-toplev/$d
 	    fi
-	  fi
-	done
-	(cd etc; $MAKE MAKEINFOFLAGS="$MAKEINFOFLAGS" info)
-	$MAKE distclean
-	mkdir proto-toplev/etc
-	(cd proto-toplev/etc;
-	    for i in $ETC_SUPPORT; do
-		ln -s ../../etc/$i .
-		done)
-	#
-	# Take out texinfo from configurable dirs
-	rm proto-toplev/configure.ac
-	sed -e '/^host_tools=/s/texinfo //' \
-	    <configure.ac >proto-toplev/configure.ac
-	#
-	mkdir proto-toplev/texinfo
-	ln -s ../../texinfo/texinfo.tex	proto-toplev/texinfo/
-	if test -r texinfo/util/tex3patch ; then
-	    mkdir proto-toplev/texinfo/util && \
-		ln -s ../../../texinfo/util/tex3patch proto-toplev/texinfo/util
-	else
-	    true
 	fi
-	chmod -R og=u . || chmod og=u `find . -print`
-	#
-	# Create .gmo files from .po files.
-	for f in `find . -name '*.po' -type f -print`; do
-	    msgfmt -o `echo $f | sed -e 's/\.po$/.gmo/'` $f
-	done
-	#
-	rm -f $package-$ver
-	ln -s proto-toplev $package-$ver
+    done
+    (cd etc; $MAKE MAKEINFOFLAGS="$MAKEINFOFLAGS" info)
+    $MAKE distclean
+    mkdir proto-toplev/etc
+    (cd proto-toplev/etc;
+	for i in $ETC_SUPPORT; do
+	    ln -s ../../etc/$i .
+	done)
+    #
+    # Take out texinfo from configurable dirs
+    rm proto-toplev/configure.ac
+    sed -e '/^host_tools=/s/texinfo //' \
+	<configure.ac >proto-toplev/configure.ac
+    #
+    mkdir proto-toplev/texinfo
+    ln -s ../../texinfo/texinfo.tex proto-toplev/texinfo/
+    if test -r texinfo/util/tex3patch ; then
+	mkdir proto-toplev/texinfo/util && \
+	    ln -s ../../../texinfo/util/tex3patch proto-toplev/texinfo/util
+    fi
+    #
+    # Create .gmo files from .po files.
+    for f in `find . -name '*.po' -type f -print`; do
+	msgfmt -o `echo $f | sed -e 's/\.po$/.gmo/'` $f
+    done
+    #
+    rm -f $package-$ver
+    ln -s proto-toplev $package-$ver
 }
 
 CVS_NAMES='-name CVS -o -name .cvsignore'
 
-# Add an md5sum to the built tarball
-do_md5sum()
+# Add a sha256sum to the built tarball
+do_sha256sum()
 {
-    echo "==> Adding md5 checksum to top-level directory"
+    echo "==> Adding sha256 checksum to top-level directory"
     (cd proto-toplev && find * -follow \( $CVS_NAMES \) -prune \
 	-o -type f -print \
-	| xargs $MD5PROG > ../md5.new)
-    rm -f proto-toplev/md5.sum
-    mv md5.new proto-toplev/md5.sum
+	| xargs $SHA256PROG > ../sha256.new)
+    rm -f proto-toplev/sha256.sum
+    mv sha256.new proto-toplev/sha256.sum
 }
 
 # Build the release tarball
@@ -186,9 +233,17 @@ do_tar()
     ver=$2
     echo "==> Making $package-$ver.tar"
     rm -f $package-$ver.tar
-    find $package-$ver -follow \( $CVS_NAMES \) -prune \
-	-o -type f -print \
-	| tar cTfh - $package-$ver.tar
+    if test x$release_date == "x" ; then
+       find $package-$ver -follow \( $CVS_NAMES \) -prune -o -type f -print \
+	   | tar cTfh - $package-$ver.tar
+    else
+	# Attempt to create a consistent, reproducible tarball using the
+	# specified date.
+	find $package-$ver -follow \( $CVS_NAMES \) -prune -o -type f -print \
+	    | LC_ALL=C sort \
+	    | tar cTfh - $package-$ver.tar \
+		  --mtime=$release_date --group=0 --owner=0
+    fi
 }
 
 # Compress the output with bzip2
@@ -228,7 +283,17 @@ do_xz()
     ver=$2
     echo "==> Xzipping $package-$ver.tar.xz"
     rm -f $package-$ver.tar.xz
-    $XZPROG -k -v -9 $package-$ver.tar
+    $XZPROG -k -v -9 -T0 $package-$ver.tar
+}
+
+# Compress the output with zstd
+do_zstd()
+{
+    package=$1
+    ver=$2
+    echo "==> Zzipping $package-$ver.tar.zst"
+    rm -f $package-$ver.tar.zst
+    $ZSTDPROG -k -v -19 -T0 $package-$ver.tar
 }
 
 # Compress the output with all selected compresion methods
@@ -247,6 +312,8 @@ do_compress()
 		do_lz $package $ver;;
 	    xz)
 		do_xz $package $ver;;
+	    zstd)
+		do_zstd $package $ver;;
 	    *)
 		echo "Unknown compression method: $comp" && exit 1;;
 	esac
@@ -276,7 +343,7 @@ tar_compress()
     verdir=${5:-$tool}
     ver=$(getver $verdir)
     do_proto_toplev $package $ver $tool "$support_files"
-    do_md5sum
+    do_sha256sum
     do_tar $package $ver
     do_compress $package $ver "$compressors"
 }
@@ -290,14 +357,14 @@ gdb_tar_compress()
     compressors=$4
     ver=$(getver $tool)
     do_proto_toplev $package $ver $tool "$support_files"
-    do_md5sum
+    do_sha256sum
     do_djunpack $package $ver
     do_tar $package $ver
     do_compress $package $ver "$compressors"
 }
 
 # The FSF "binutils" release includes gprof and ld.
-BINUTILS_SUPPORT_DIRS="bfd gas include libiberty libctf opcodes ld elfcpp gold gprof intl setup.com makefile.vms cpu zlib"
+BINUTILS_SUPPORT_DIRS="libsframe bfd gas include libiberty libctf opcodes ld elfcpp gold gprof gprofng setup.com makefile.vms cpu zlib"
 binutils_release()
 {
     compressors=$1
@@ -306,7 +373,7 @@ binutils_release()
     tar_compress $package $tool "$BINUTILS_SUPPORT_DIRS" "$compressors"
 }
 
-GAS_SUPPORT_DIRS="bfd include libiberty opcodes intl setup.com makefile.vms zlib"
+GAS_SUPPORT_DIRS="bfd include libiberty opcodes setup.com makefile.vms zlib"
 gas_release()
 {
     compressors=$1
@@ -315,7 +382,7 @@ gas_release()
     tar_compress $package $tool "$GAS_SUPPORT_DIRS" "$compressors"
 }
 
-GDB_SUPPORT_DIRS="bfd include libiberty opcodes readline sim intl libdecnumber cpu zlib contrib gnulib"
+GDB_SUPPORT_DIRS="libsframe bfd include libiberty libctf opcodes readline sim libdecnumber cpu zlib contrib gnulib gdbsupport gdbserver libbacktrace"
 gdb_release()
 {
     compressors=$1
@@ -325,7 +392,7 @@ gdb_release()
 }
 
 # Corresponding to the CVS "sim" module.
-SIM_SUPPORT_DIRS="bfd opcodes libiberty include intl gdb/version.in gdb/common/create-version.sh makefile.vms zlib"
+SIM_SUPPORT_DIRS="libsframe bfd opcodes libiberty libctf/swap.h include gdb/version.in gdb/common/create-version.sh makefile.vms zlib gnulib"
 sim_release()
 {
     compressors=$1
@@ -342,6 +409,8 @@ usage()
     echo "  -g: Compress with gzip"
     echo "  -l: Compress with lzip"
     echo "  -x: Compress with xz"
+    echo "  -z: Compress with zstd"
+    echo "  -r <date>: Create a reproducible tarball using <date> as the mtime"
     exit 1
 }
 
@@ -365,7 +434,7 @@ build_release()
 
 compressors=""
 
-while getopts ":bglx" opt; do
+while getopts ":bglr:xz" opt; do
     case $opt in
 	b)
 	    compressors="$compressors bz2";;
@@ -373,11 +442,15 @@ while getopts ":bglx" opt; do
 	    compressors="$compressors gz";;
 	l)
 	    compressors="$compressors lz";;
+	r)
+	    release_date=$OPTARG;;
 	x)
 	    compressors="$compressors xz";;
+	z)
+	    compressors="$compressors zstd";;
 	\?)
 	    echo "Invalid option: -$OPTARG" && usage;;
-  esac
+    esac
 done
 shift $((OPTIND -1))
 release=$1
